import { HttpService } from "@nestjs/axios";
import { Observable, Subscriber } from "rxjs";
import { APAccountProfile } from "../entities/APAccountProfile";
import { ConvertedResult } from "../entities/ConvertedResult";
import { DataServiceMessage } from "../entities/DataService.entity";
import { SummaryResponse } from "../entities/DescendentDataServiceArgument.entity";
import { StatusResponse } from "../entities/StatusResponse";
import { FisCreateMessageUtility, Query, DataTypeProfile, MessageTypeProfile, ResponseMessage,
        FisReadDataUtility, GenericFisData, 
        GenericFisRowColumn } from "../message/fisappmessagejsutility/interface/export";

import { State } from "./state";

export class CommonDataService {

    public tempData: any;
    // static ResponseService: FisUASRequestMessage;
    static ResponseService:FisCreateMessageUtility = new FisCreateMessageUtility("FisAppID/Name")
    public ucpId: string;
    public CurrentState: string = State.Initialisation.Code

    public currentData: Record<string, any> = {};
    static data: Record<string, any> = {};
    static className: string;
    public dataName: string = "DataService";

    constructor(public http: HttpService, ucpId: string, data = CommonDataService.data) {
        // Set initial parameters
        this.http = http;
        this.ucpId = ucpId;
        this.currentData = data;
        this.CurrentState = State.Ready.Code; // Start as ready


    }

    getDataFromServer(http: HttpService, ucpId: string, data):Observable<any>{
        return this.httpRequest(http, ucpId, data)
    }

    getData(className, filter, returnFormatType):any {
        // return new Promise((resolve, reject) => {
        return new Observable(subscriber=>{

            let message: string;
            let result: [] | string;
            // try {

            this.getDataWithStatus(className,filter, returnFormatType, 1).subscribe({
                next: (res) => {
                    result = res;
                    subscriber.next(result)
                },
                error: (e)=>{
                    subscriber.error(e)
                }

            })
        })
    }

    

    getDataWithType(filter, className:string, message:any, returnFormatType:string, data:any[]): any {


            return new Observable((subscriber)=>{

                let key = []
                let value = []
                if (filter) { //check whether filter existed
                    let semicolumn = filter.split(";"); //separate the semicolumn
                    for (let i = 0; i < semicolumn.length; i++) {
                        key[i] = semicolumn[i].split("=")[0];
                        value[i] = semicolumn[i].split("=")[1];
                    }
                }

                this.getDataFromServer(this.http, this.ucpId, this.currentData).subscribe({
                // .then((res: ResponseMessage)=> {
                    next: (res: ResponseMessage)=>{
                    const dataUtil:FisReadDataUtility = new FisReadDataUtility();
                    const messageType: MessageTypeProfile = new MessageTypeProfile();
                    const dataType:DataTypeProfile = new DataTypeProfile();


                    // const readData = dataUtil.readDataType(res, referenceType);
                    let genericFisData:GenericFisData;
                    let result:GenericFisRowColumn[];

                    let readData:any;

                    if(res.data[0].data && res.data[0].data['GenericFisData'])
                    {
                        readData = dataUtil.readDataType(res.data[0].header.requestMessageRespondTo, res.data[0], messageType,dataType)
                    }else{
                        dataType.name = "GenericFisData";
                        readData = res;
                    }


                    if(dataType.name == "GenericFisData"){
                        genericFisData = readData as GenericFisData;

                        result = genericFisData.data[this.dataName].rows.row.filter(val => {

                                    if (filter) {
                                        for (let i = 0; i < key.length; i++) {
                                            if (val.column[key[i]] != value[i]) {
                                                return false;
                                            }
                                        }

                                    }
                                    return true

                                }).map(x => x.column);
                        

                        data = this.chunkFormatType(returnFormatType, result, 0);
                                
                        let returnMessage:DataServiceMessage[] = message;

                        let summaryMessage:SummaryResponse = new SummaryResponse();

                            summaryMessage.className = className;
                            summaryMessage.filter = filter;
                            summaryMessage.returnFormatType = returnFormatType;
                            summaryMessage.returnSequenceTotal = result.length;

                        let statusResponse = new StatusResponse();
                        statusResponse.status = "1";


                        let dataReturn:APAccountProfile[] = data;

                        message = [
                            summaryMessage,
                            dataReturn,
                            statusResponse,
                        ];

                        let messageName = className.replace('DataService','') + "Message";

                        subscriber.next({
                            StatusResponse: statusResponse,
                            SummaryResponse: summaryMessage,
                            [messageName]: {
                                data:dataReturn
                            }
                        });

                        subscriber.complete()

                    }
                }
                })


            })


    }

    getDataWithStatus(className: string,
        filter: string,
        returnFormatType: string,
        gq, chunkSize = null): Observable<any> {
        // return new Promise((resolve, reject) => {
            return new Observable((subscriber)=>{

            let JSONstring: string;
            let convertedResult: DataServiceMessage;

            let error = null;
            let result = [];
            // try {

                if (this.CurrentState = State.Initialisation.Code) {
                    
                    JSONstring = State.Initialisation.Message

                }

                if (this.CurrentState = State.Busy.Code) {
                    
                        
                    JSONstring = State.Busy.Message
                }

                if (this.CurrentState = State.Ready.Code) {

                    // Retrieve new data from server
                    this.getDataFromServer(this.http, this.ucpId, this.currentData).subscribe({
                        next: (res)=>{
                    
                    // .then(res => {
                        if (res == null) // Check if it is null
                        {
                            
                            JSONstring = State.Error.Message + "No data entity in cache to filter"
                                
                        }
                        else {

                            if (res['data'] === undefined ||
                                res['data'].DataService === undefined ||
                                res['data'].DataService.rows['row'] === undefined
                            ) { //check whether row entity exist before perform filter
                                
                                JSONstring =  State.Error.Message + " No row entity in filter"
                            } else {

                                let key = []
                                let value = []
                                if (filter) { //check whether filter existed

                                    let semicolumn = filter.split(";"); //separate the semicolumn


                                    for (let i = 0; i < semicolumn.length; i++) {

                                        key[i] = semicolumn[i].split("=")[0];
                                        value[i] = semicolumn[i].split("=")[1];

                                    }
                                }

                               if (res['data'].DataService.rows.row.length > 0) // check whether got any record returned
                                {


                                    result = res['data'].DataService.rows.row.filter(function (v) {
                                        let check = true;

                                        if (filter) {

                                            for (let i = 0; i < key.length; i++) {

                                                if (v.column[key[i]] === undefined) { // check whether column exist before proceed
                                                    error = State.Error.Message + " No Column for corresponding key " + [key[i]] + ".";
                                                    return false;
                                                } else if (v.column[key[i]] != value[i]) {
                                                    return false;

                                                }

                                            }
                                        }
                                        return check;
                                    })

                                    if (result.length === 0) { //show error if no record found in the filter 
                                        error = State.Error.Message + " no record found in filter";
                                    }

                                } else {//if there got no record returned from backend

                                    error = State.Error.Message + " no record returned";
                                }


                                if (!error) {

                                    if (gq == 1) { // result return to graphql 
                                        result = this.chunkFormatType(returnFormatType, result);
                                        convertedResult = new ConvertedResult(
                                            className,
                                            JSON.stringify(result[0]),
                                            filter,
                                            returnFormatType,
                                            result.length,
                                            1
                                        );
                                        subscriber.next(convertedResult)
                                    } else { //result return to subscriberProxy

                                        // result.map((el, idx) => { //will be use to subscribe multiple observable from back office
                                            convertedResult = new ConvertedResult(
                                                className,
                                                result[0],
                                                filter,
                                                returnFormatType,
                                                result.length,
                                                result.length + 1
                                            );

                                            // res.push(convertedResult)
                                        // })
                                        subscriber.next(convertedResult)
                                    }



                                } else { // resolve the error for no record and filter with no record

                                    convertedResult = new ConvertedResult(
                                        className,
                                        error,
                                        filter,
                                        returnFormatType
                                    );
                                    subscriber.next(convertedResult)
                                }

                            }


                        }

                        convertedResult = new ConvertedResult(
                            className,
                            JSONstring,
                            filter,
                            returnFormatType
                        );

                        subscriber.next(JSONstring);
                        }
                    })
                }
            // } catch (e) {
                subscriber.error(e=>{

                    return convertedResult = new ConvertedResult(
                        className,
                        State.Error.Message + e.Message,
                        filter,
                        returnFormatType
                    );

                })

                // reject(convertedResult);
            // }

        })
    }




    public httpRequest(http, ucpId, data: string):Observable<any> {
        if (!CommonDataService.ResponseService) {
            CommonDataService.ResponseService = new FisCreateMessageUtility('getQueryMessage');
        }

        try {

            return http.post(process.env.UCP_HOST, CommonDataService.ResponseService.getQueryMessage(ucpId,Query.GetData, data))
         
        } catch (e) {
            console.log(e)
            throw e;
        }
    }

    public chunkFormatType(type, data, block = null) {

        let chunkSize = 0;

        if (type === "byRow") {
            chunkSize = 1;
        } else if (type === "byBatchOf5") {
            chunkSize = 5;
        } else if (type === "byBatchOf10") {
            chunkSize = 10;
        }

        if (chunkSize == 1) {
            return [data[0]]
        } else if (chunkSize > 1) { // record by batch of #
            let result = this.chunk(data, chunkSize);

            return block != null ? result[block]: result;
        } else { //record return by FULL
            return data;
        }
    }

    public chunk(arr, len) {
        let chunks = [],
            i = 0,
            n = arr.length;

        while (i < n) {
            chunks.push(arr.slice(i, i += len));
        }

        return chunks;
    }

    public validate(key,value):string
    {
        return "";
    }
}