import { ArgsType,  Field,  Int,  ObjectType } from '@nestjs/graphql';
import { SummaryResponse } from './DescendentDataServiceArgument.entity';
import { Document as DocumentType } from '../message/fisappmessagejsutility/interface/export'
import { CustomFilter } from './CustomFilter.validate';
import { CustomReturnFormatType } from './CustomReturnFormatType.validate';
import { Validate } from 'class-validator'; 
import { StatusResponse } from './StatusResponse';

export const thisClassName = "DocumentDataService";

@ObjectType()
@ArgsType() 
export class DocumentArgument {

    public className: string = thisClassName; 

    @Field({ nullable: true })
    @Validate(CustomFilter)
    filter?: string; 

    @Field({ nullable: true })
    @Validate(CustomReturnFormatType)
    returnFormatType?: string;

}


@ObjectType()
@ArgsType() 
export class DocumentMessage  {

    @Field(type => [Document]) 
    data?: Document[]; 

}


@ObjectType()
@ArgsType() 
export class Document implements DocumentType {
    
    @Field({ nullable: true})
    subs_code: string 
    
    @Field({ nullable: true})
    doc_catagory: string 

    @Field(type=> Int, { nullable: true})
    doc_id: number 

    @Field(type=> Int, { nullable: true})
    doc_type_id: number 

    @Field({ nullable: true})
    doc_category: string 

    @Field({ nullable: true})
    doc_by_org_code: string 

    @Field(type=> Int, { nullable: true})
    acct_id: number 

    @Field({ nullable: true})
    doc_post_type: string 

    @Field({ nullable: true})
    doc_update_type: string 

    @Field({ nullable: true})
    doc_ref: string 

    @Field({ nullable: true})
    doc_dt: string 

    @Field({ nullable: true})
    doc_tot_entry: string 

    @Field({ nullable: true})
    doc_tot_debit: string 

    @Field({ nullable: true})
    doc_tot_credit: string 

    @Field({ nullable: true})
    doc_status: string 

    @Field(type=> Int, { nullable: true})
    tran_doc_periodid: number 

    @Field({ nullable: true})
    doc_desc: string 

    @Field({ nullable: true})
    doc_remarks: string 

    @Field(type=> Int, { nullable: true})
    doc_by_org_id: number 

    @Field({ nullable: true})
    doc_currency: string 

    @Field(type=> Int, { nullable: true})
    doc_currency_rate: number 

    @Field({ nullable: true})
    doc_post_dt: string 

    @Field(type=> Int, { nullable: true})
    prof_id: number 

    @Field({ nullable: true})
    doc_pay_terms: string 
}


@ObjectType()
@ArgsType() 
export class DocumentUnion {

    @Field(type=> SummaryResponse, { nullable: true})
    SummaryResponse?:SummaryResponse;

    @Field(type=> StatusResponse, { nullable: true})
    StatusResponse?:StatusResponse;

    @Field(type=> DocumentMessage, { nullable: true})
    DocumentMessage?:DocumentMessage
}