import { ArgsType,  createUnionType,  Field,  Float,  Int,  ObjectType } from '@nestjs/graphql';
import { DescendentDataServiceArgument,  SummaryResponse } from './DescendentDataServiceArgument.entity';
import { ProductProfile as ProductProfileType } from '../message/fisappmessagejsutility/interface/export';
import { CustomFilter } from './CustomFilter.validate';
import { CustomReturnFormatType } from './CustomReturnFormatType.validate';
import { Validate } from 'class-validator'
import { StatusResponse } from './StatusResponse';


export const thisClassName = "ProductProfileDataService";

@ObjectType()
@ArgsType() 
export class ProductProfileArgument {

    className: string = thisClassName;

    
    @Field({ nullable: true })
    @Validate(CustomFilter)
    filter?: string; 

    @Field({ nullable: true })
    @Validate(CustomReturnFormatType)
    returnFormatType?: string;
 
}


@ObjectType()
@ArgsType() 
export class ProductProfileMessage {

    @Field(type => [ProductProfile]) 
    data?: ProductProfile[]; 
}


@ObjectType()
@ArgsType() 
export class ProductProfile implements ProductProfileType{
    
    @Field(type=> Int,{ nullable: true})
    prd_id:number;

    @Field(type=> Int,{ nullable: true})
    prd_ref_id:number;

    @Field({ nullable: true})
    prd_type: string;

    @Field(type=> Int,{ nullable: true})
    loc_id:number;

    @Field(type=> Int,{ nullable: true})
    orgn_id:number;

    @Field(type=> Int,{ nullable: true})
    prof_id:number;

    @Field({ nullable: true})
    prd_code_alias: string;

    @Field(type=> Int,{ nullable: true})
    prd_qty:number;

    @Field({ nullable: true})
    prd_unit_cost:string;

    @Field({ nullable: true})
    prd_date_pur: string;

    @Field({ nullable: true})
    prd_model_no: string;

    @Field({ nullable: true})
    prd_man_id: string;

    @Field(type=> Int,{ nullable: true})
    prd_dist_class_id:number;

    @Field({ nullable: true})
    prd_is_primary: string;

    @Field(type=> Int,{ nullable: true})
    product_ref_prd_ref_id:number;

    @Field({ nullable: true})
    prd_ref_type: string;

    @Field({ nullable: true})
    prd_ref_unit_cost:string;

    @Field({ nullable: true})
    prd_ref_list_price:string;

    @Field(type=> Int,{ nullable: true})
    prd_ref_prof_id:number;

    @Field(type=> Int,{ nullable: true})
    prd_ref_prd_dist_class_id:number;

    @Field({ nullable: true})
    prd_ref_prd_assign_serial_no: string;

    @Field({ nullable: true})
    orgn_profile_orgn_short_name: string;

    @Field({ nullable: true})
    prd_code:string;

    @Field({ nullable: true})
    prd_assign_serial_no: string;

    @Field({ nullable: true})
    prd_ref_code:string;

    @Field({ nullable: true})
    orgn_full_name: string;

    @Field({ nullable: true})
    orgn_code:string;

    @Field({ nullable: true})
    loc_code:string;

    @Field({ nullable: true})
    loc_desc: string;

    @Field({ nullable: true})
    prd_ref_desc: string;

    @Field({ nullable: true})
    prd_list_price:string;

    @Field({ nullable: true})
    prd_ref_uom: string;

    @Field({ nullable: true})
    prd_status_code: string;

    @Field({ nullable: true})
    prd_brand_code: string;

    @Field({ nullable: true})
    prd_origin_country_code: string;

    @Field({ nullable: true})
    bin_code: string;

    @Field({ nullable: true})
    prd_desc: string
   
}


@ObjectType()
@ArgsType() 
export class ProductProfileUnion {

    @Field(type=> SummaryResponse, { nullable: true})
    SummaryResponse?:SummaryResponse;

    @Field(type=> StatusResponse, { nullable: true})
    StatusResponse?:StatusResponse;

    @Field(type=> ProductProfileMessage, { nullable: true})
    ProductProfileMessage?:ProductProfileMessage
}